@@ -84,9 +84,10 @@ class LibevConnection(BaseConnection):
         :param pika.connection.Parameters parameters: Connection parameters
         :param on_open_callback: The method to call when the connection is open
         :type on_open_callback: method
-        :param on_open_error_callback: Method to call if the connection cannot
-                                       be opened
-        :type on_open_error_callback: method
+        :param method on_open_error_callback: Called if the connection can't
+            be established: on_open_error_callback(connection, str|exception)
+        :param method on_close_callback: Called when the connection is closed:
+            on_close_callback(connection, reason_code, reason_text)
         :param bool stop_ioloop_on_close: Call ioloop.stop() if disconnected
         :param custom_ioloop: Override using the default IOLoop in libev
         :param on_signal_callback: Method to call if SIGINT or SIGTERM occur
@@ -99,6 +100,7 @@ class LibevConnection(BaseConnection):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", RuntimeWarning)
                 self.ioloop = pyev.default_loop()
+                self.ioloop.update()
 
         self.async = None
         self._on_signal_callback = on_signal_callback
@@ -127,7 +129,7 @@ class LibevConnection(BaseConnection):
             if self._on_signal_callback and not global_sigterm_watcher:
                 global_sigterm_watcher = \
                     self.ioloop.signal(signal.SIGTERM,
-                                                                                  self._handle_sigterm)
+                                       self._handle_sigterm)
 
             if self._on_signal_callback and not global_sigint_watcher:
                 global_sigint_watcher = self.ioloop.signal(signal.SIGINT,
@@ -136,11 +138,14 @@ class LibevConnection(BaseConnection):
             if not self._io_watcher:
                 self._io_watcher = \
                     self.ioloop.io(self.socket.fileno(),
-                                                                        self._PIKA_TO_LIBEV_ARRAY[self.event_state],
-                                                                        self._handle_events)
+                                   self._PIKA_TO_LIBEV_ARRAY[self.event_state],
+                                   self._handle_events)
 
+            # NOTE: if someone knows why this async is needed here, please add
+            # a comment in the code that explains it.
             self.async = pyev.Async(self.ioloop, self._noop_callable)
             self.async.start()
+
             if self._on_signal_callback:
                 global_sigterm_watcher.start()
             if self._on_signal_callback:
@@ -158,7 +163,8 @@ class LibevConnection(BaseConnection):
         be wiped.
 
         """
-        for timer in self._active_timers.keys():
+        active_timers = list(self._active_timers.keys())
+        for timer in active_timers:
             self.remove_timeout(timer)
         if global_sigint_watcher:
             global_sigint_watcher.stop()
@@ -196,7 +202,7 @@ class LibevConnection(BaseConnection):
 
     def _reset_io_watcher(self):
         """Reset the IO watcher; retry as necessary
-        
+
         """
         self._io_watcher.stop()
 
@@ -208,8 +214,9 @@ class LibevConnection(BaseConnection):
                     self._PIKA_TO_LIBEV_ARRAY[self.event_state])
 
                 break
-            except:  # sometimes the stop() doesn't complete in time
-                if retries > 5: raise
+            except Exception:  # sometimes the stop() doesn't complete in time
+                if retries > 5:
+                    raise
                 self._io_watcher.stop()  # so try it again
                 retries += 1
 
@@ -235,12 +242,12 @@ class LibevConnection(BaseConnection):
             (callback_method, callback_timeout,
              kwargs) = self._active_timers[timer]
 
+            self.remove_timeout(timer)
+
             if callback_timeout:
                 callback_method(timeout=timer, **kwargs)
             else:
                 callback_method(**kwargs)
-
-            self.remove_timeout(timer)
         else:
             LOGGER.warning('Timer callback_method not found')
 
@@ -267,7 +274,7 @@ class LibevConnection(BaseConnection):
         :rtype: timer instance handle.
 
         """
-        LOGGER.debug('deadline: {0}'.format(deadline))
+        LOGGER.debug('deadline: %s', deadline)
         timer = self._get_timer(deadline)
         self._active_timers[timer] = (callback_method, callback_timeout,
                                       callback_kwargs)
@@ -282,9 +289,13 @@ class LibevConnection(BaseConnection):
 
         """
         LOGGER.debug('stop')
-        self._active_timers.pop(timer, None)
-        timer.stop()
-        self._stopped_timers.append(timer)
+        try:
+            self._active_timers.pop(timer)
+        except KeyError:
+            LOGGER.warning("Attempted to remove inactive timer %s", timer)
+        else:
+            timer.stop()
+            self._stopped_timers.append(timer)
 
     def _create_and_connect_to_socket(self, sock_addr_tuple):
         """Call super and then set the socket to nonblocking."""
