@@ -11,6 +11,7 @@ except ImportError:
 import uuid
 
 from forward_server import ForwardServer
+from test_utils import retry_assertion
 
 import pika
 from pika.adapters import blocking_connection
@@ -33,7 +34,9 @@ import pika.exceptions
 # Disable warning Invalid variable name
 # pylint: disable=C0103
 
+
 LOGGER = logging.getLogger(__name__)
+
 PARAMS_URL_TEMPLATE = (
     'amqp://guest:guest@127.0.0.1:%(port)s/%%2f?socket_timeout=1')
 DEFAULT_URL = PARAMS_URL_TEMPLATE % {'port': 5672}
@@ -41,6 +44,11 @@ DEFAULT_PARAMS = pika.URLParameters(DEFAULT_URL)
 DEFAULT_TIMEOUT = 15
 
 
+
+def setUpModule():
+    logging.basicConfig(level=logging.DEBUG)
+
+
 class BlockingTestCaseBase(unittest.TestCase):
 
     TIMEOUT = DEFAULT_TIMEOUT
@@ -65,6 +73,14 @@ class BlockingTestCaseBase(unittest.TestCase):
         LOGGER.info('%s TIMED OUT (%s)', datetime.utcnow(), self)
         self.fail('Test timed out')
 
+    @retry_assertion(TIMEOUT/2)
+    def _assert_exact_message_count_with_retries(self,
+                                                 channel,
+                                                 queue,
+                                                 expected_count):
+        frame = channel.queue_declare(queue, passive=True)
+        self.assertEqual(frame.method.message_count, expected_count)
+
 
 class TestCreateAndCloseConnection(BlockingTestCaseBase):
 
@@ -82,6 +98,25 @@ class TestCreateAndCloseConnection(BlockingTestCaseBase):
         self.assertFalse(connection.is_closing)
 
 
+class TestMultiCloseConnection(BlockingTestCaseBase):
+
+    def test(self):
+        """BlockingConnection: Close connection twice"""
+        connection = self._connect()
+        self.assertIsInstance(connection, pika.BlockingConnection)
+        self.assertTrue(connection.is_open)
+        self.assertFalse(connection.is_closed)
+        self.assertFalse(connection.is_closing)
+
+        connection.close()
+        self.assertTrue(connection.is_closed)
+        self.assertFalse(connection.is_open)
+        self.assertFalse(connection.is_closing)
+
+        # Second close call shouldn't crash
+        connection.close()
+
+
 class TestConnectionContextManagerClosesConnection(BlockingTestCaseBase):
     def test(self):
         """BlockingConnection: connection context manager closes connection"""
@@ -94,7 +129,7 @@ class TestConnectionContextManagerClosesConnection(BlockingTestCaseBase):
 
 class TestConnectionContextManagerClosesConnectionAndPassesOriginalException(BlockingTestCaseBase):
     def test(self):
-        """BlockingConnection: connection context manager closes connection and passes original exception"""
+        """BlockingConnection: connection context manager closes connection and passes original exception"""  # pylint: disable=C0301
         class MyException(Exception):
             pass
 
@@ -109,7 +144,7 @@ class TestConnectionContextManagerClosesConnectionAndPassesOriginalException(Blo
 
 class TestConnectionContextManagerClosesConnectionAndPassesSystemException(BlockingTestCaseBase):
     def test(self):
-        """BlockingConnection: connection context manager closes connection and passes system exception"""
+        """BlockingConnection: connection context manager closes connection and passes system exception"""  # pylint: disable=C0301
         with self.assertRaises(SystemExit):
             with self._connect() as connection:
                 self.assertTrue(connection.is_open)
@@ -178,7 +213,10 @@ class TestSuddenBrokerDisconnectBeforeChannel(BlockingTestCaseBase):
     def test(self):
         """BlockingConnection resets properly on TCP/IP drop during channel()
         """
-        with ForwardServer((DEFAULT_PARAMS.host, DEFAULT_PARAMS.port)) as fwd:
+        with ForwardServer(
+            remote_addr=(DEFAULT_PARAMS.host, DEFAULT_PARAMS.port),
+            local_linger_args=(1, 0)) as fwd:
+
             self.connection = self._connect(
                 PARAMS_URL_TEMPLATE % {"port": fwd.server_address[1]})
 
@@ -198,7 +236,10 @@ class TestNoAccessToFileDescriptorAfterConnectionClosed(BlockingTestCaseBase):
     def test(self):
         """BlockingConnection no access file descriptor after ConnectionClosed
         """
-        with ForwardServer((DEFAULT_PARAMS.host, DEFAULT_PARAMS.port)) as fwd:
+        with ForwardServer(
+            remote_addr=(DEFAULT_PARAMS.host, DEFAULT_PARAMS.port),
+            local_linger_args=(1, 0)) as fwd:
+
             self.connection = self._connect(
                 PARAMS_URL_TEMPLATE % {"port": fwd.server_address[1]})
 
@@ -244,7 +285,10 @@ class TestDisconnectDuringConnectionStart(BlockingTestCaseBase):
     def test(self):
         """ BlockingConnection TCP/IP connection loss in CONNECTION_START
         """
-        fwd = ForwardServer((DEFAULT_PARAMS.host, DEFAULT_PARAMS.port))
+        fwd = ForwardServer(
+            remote_addr=(DEFAULT_PARAMS.host, DEFAULT_PARAMS.port),
+            local_linger_args=(1, 0))
+
         fwd.start()
         self.addCleanup(lambda: fwd.stop() if fwd.running else None)
 
@@ -267,7 +311,9 @@ class TestDisconnectDuringConnectionTune(BlockingTestCaseBase):
     def test(self):
         """ BlockingConnection TCP/IP connection loss in CONNECTION_TUNE
         """
-        fwd = ForwardServer((DEFAULT_PARAMS.host, DEFAULT_PARAMS.port))
+        fwd = ForwardServer(
+            remote_addr=(DEFAULT_PARAMS.host, DEFAULT_PARAMS.port),
+            local_linger_args=(1, 0))
         fwd.start()
         self.addCleanup(lambda: fwd.stop() if fwd.running else None)
 
@@ -290,7 +336,10 @@ class TestDisconnectDuringConnectionProtocol(BlockingTestCaseBase):
     def test(self):
         """ BlockingConnection TCP/IP connection loss in CONNECTION_PROTOCOL
         """
-        fwd = ForwardServer((DEFAULT_PARAMS.host, DEFAULT_PARAMS.port))
+        fwd = ForwardServer(
+            remote_addr=(DEFAULT_PARAMS.host, DEFAULT_PARAMS.port),
+            local_linger_args=(1, 0))
+
         fwd.start()
         self.addCleanup(lambda: fwd.stop() if fwd.running else None)
 
@@ -327,7 +376,7 @@ class TestProcessDataEvents(BlockingTestCaseBase):
         self.assertLess(elapsed, 0.25)
 
 
-class TestConnectionBlockAndUnblock(BlockingTestCaseBase):
+class TestConnectionRegisterForBlockAndUnblock(BlockingTestCaseBase):
 
     def test(self):
         """BlockingConnection register for Connection.Blocked/Unblocked"""
@@ -358,6 +407,35 @@ class TestConnectionBlockAndUnblock(BlockingTestCaseBase):
         self.assertEqual(unblocked_buffer, ["unblocked"])
 
 
+class TestBlockedConnectionTimeout(BlockingTestCaseBase):
+
+    def test(self):
+        """BlockingConnection Connection.Blocked timeout """
+        url = DEFAULT_URL + '&blocked_connection_timeout=0.001'
+        conn = self._connect(url=url)
+
+        # NOTE: I haven't figured out yet how to coerce RabbitMQ to emit
+        # Connection.Block and Connection.Unblock from the test, so we'll
+        # simulate it for now
+
+        # Simulate Connection.Blocked
+        conn._impl._on_connection_blocked(pika.frame.Method(
+            0,
+            pika.spec.Connection.Blocked('TestBlockedConnectionTimeout')))
+
+        # Wait for connection teardown
+        with self.assertRaises(pika.exceptions.ConnectionClosed) as excCtx:
+            while True:
+                conn.process_data_events(time_limit=1)
+
+        self.assertEqual(
+            excCtx.exception.args,
+            (pika.connection.InternalCloseReasons.BLOCKED_CONNECTION_TIMEOUT,
+             'Blocked connection timeout expired'))
+
+
+
+
 class TestAddTimeoutRemoveTimeout(BlockingTestCaseBase):
 
     def test(self):
@@ -415,7 +493,7 @@ class TestRemoveTimeoutFromTimeoutCallback(BlockingTestCaseBase):
         while not rx_timer2:
             connection.process_data_events(time_limit=None)
 
-        self.assertNotIn(timer_id1, connection._impl.ioloop._timeouts)
+        self.assertNotIn(timer_id1, connection._impl.ioloop._poller._timeouts)
         self.assertFalse(connection._ready_events)
 
 
@@ -557,8 +635,9 @@ class TestExchangeBindAndUnbind(BlockingTestCaseBase):
                    mandatory=True)
 
         # Check that the queue now has one message
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 1)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=1)
 
         # Unbind the exchanges
         frame = ch.exchange_unbind(destination=dest_exg_name,
@@ -740,10 +819,9 @@ class TestBasicGet(BlockingTestCaseBase):
         LOGGER.info('%s ACKED (%s)', datetime.utcnow(), self)
 
         # Verify that the queue is now empty
-        frame = ch.queue_declare(q_name, passive=True)
-        LOGGER.info('%s DECLARE PASSIVE QUEUE DONE (%s)',
-                    datetime.utcnow(), self)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestBasicReject(BlockingTestCaseBase):
@@ -785,8 +863,9 @@ class TestBasicReject(BlockingTestCaseBase):
 
         # Verify that exactly one message is present in the queue, namely the
         # second one
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 1)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=1)
         (rx_method, _, rx_body) = ch.basic_get(q_name, no_ack=False)
         self.assertEqual(rx_body, as_bytes('TestBasicReject2'))
 
@@ -831,8 +910,9 @@ class TestBasicRejectNoRequeue(BlockingTestCaseBase):
         ch.basic_reject(rx_method.delivery_tag, requeue=False)
 
         # Verify that no messages are present in the queue
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestBasicNack(BlockingTestCaseBase):
@@ -874,8 +954,9 @@ class TestBasicNack(BlockingTestCaseBase):
 
         # Verify that exactly one message is present in the queue, namely the
         # second one
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 1)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=1)
         (rx_method, _, rx_body) = ch.basic_get(q_name, no_ack=False)
         self.assertEqual(rx_body, as_bytes('TestBasicNack2'))
 
@@ -920,8 +1001,9 @@ class TestBasicNackNoRequeue(BlockingTestCaseBase):
         ch.basic_nack(rx_method.delivery_tag, requeue=False)
 
         # Verify that no messages are present in the queue
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestBasicNackMultiple(BlockingTestCaseBase):
@@ -964,8 +1046,9 @@ class TestBasicNackMultiple(BlockingTestCaseBase):
         ch.basic_nack(rx_method.delivery_tag, multiple=True, requeue=True)
 
         # Verify that both messages are present in the queue
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 2)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
         (rx_method, _, rx_body) = ch.basic_get(q_name, no_ack=False)
         self.assertEqual(rx_body,
                          as_bytes('TestBasicNackMultiple1'))
@@ -1365,8 +1448,7 @@ class TestBasicPublishDeliveredWhenPendingUnroutable(BlockingTestCaseBase):
         self.addCleanup(self._connect().channel().queue_delete, q_name)
 
         # Bind the queue to the exchange using routing key
-        frame = ch.queue_bind(q_name, exchange=exg_name,
-                              routing_key=routing_key)
+        ch.queue_bind(q_name, exchange=exg_name, routing_key=routing_key)
 
         # Attempt to send an unroutable message in the queue via basic_publish
         res = ch.basic_publish(exg_name, routing_key='',
@@ -1384,11 +1466,9 @@ class TestBasicPublishDeliveredWhenPendingUnroutable(BlockingTestCaseBase):
         self.assertEqual(res, True)
 
         # Wait for the queue to get the routable message
-        while ch.queue_declare(q_name, passive=True).method.message_count < 1:
-            pass
-
-        self.assertEqual(
-            ch.queue_declare(q_name, passive=True).method.message_count, 1)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=1)
 
         msg = ch.basic_get(q_name)
 
@@ -1411,8 +1491,9 @@ class TestBasicPublishDeliveredWhenPendingUnroutable(BlockingTestCaseBase):
         ch.basic_ack(delivery_tag=rx_method.delivery_tag, multiple=False)
 
         # Verify that the queue is now empty
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestPublishAndConsumeWithPubacksAndQosOfOne(BlockingTestCaseBase):
@@ -1443,8 +1524,7 @@ class TestPublishAndConsumeWithPubacksAndQosOfOne(BlockingTestCaseBase):
         self.addCleanup(self._connect().channel().queue_delete, q_name)
 
         # Bind the queue to the exchange using routing key
-        frame = ch.queue_bind(q_name, exchange=exg_name,
-                              routing_key=routing_key)
+        ch.queue_bind(q_name, exchange=exg_name, routing_key=routing_key)
 
         # Deposit a message in the queue via basic_publish
         msg1_headers = dict(
@@ -1531,8 +1611,9 @@ class TestPublishAndConsumeWithPubacksAndQosOfOne(BlockingTestCaseBase):
         ch.basic_ack(delivery_tag=rx_method.delivery_tag, multiple=False)
 
         # Verify that the queue is now empty
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
         # Attempt to cosume again with a short timeout
         connection.process_data_events(time_limit=0.005)
@@ -1549,6 +1630,111 @@ class TestPublishAndConsumeWithPubacksAndQosOfOne(BlockingTestCaseBase):
         self.assertEqual(frame.method.consumer_tag, consumer_tag)
 
 
+class TestTwoBasicConsumersOnSameChannel(BlockingTestCaseBase):
+
+    def test(self):  # pylint: disable=R0914
+        """BlockingChannel: two basic_consume consumers on same channel
+        """
+        connection = self._connect()
+
+        ch = connection.channel()
+
+        exg_name = 'TestPublishAndConsumeAndQos_exg_' + uuid.uuid1().hex
+        q1_name = 'TestTwoBasicConsumersOnSameChannel_q1' + uuid.uuid1().hex
+        q2_name = 'TestTwoBasicConsumersOnSameChannel_q2' + uuid.uuid1().hex
+        q1_routing_key = 'TestTwoBasicConsumersOnSameChannel1'
+        q2_routing_key = 'TestTwoBasicConsumersOnSameChannel2'
+
+        # Place channel in publisher-acknowledgments mode so that publishing
+        # with mandatory=True will be synchronous
+        ch.confirm_delivery()
+
+        # Declare a new exchange
+        ch.exchange_declare(exg_name, exchange_type='direct')
+        self.addCleanup(connection.channel().exchange_delete, exg_name)
+
+        # Declare the two new queues and bind them to the exchange
+        ch.queue_declare(q1_name, auto_delete=True)
+        self.addCleanup(self._connect().channel().queue_delete, q1_name)
+        ch.queue_bind(q1_name, exchange=exg_name, routing_key=q1_routing_key)
+
+        ch.queue_declare(q2_name, auto_delete=True)
+        self.addCleanup(self._connect().channel().queue_delete, q2_name)
+        ch.queue_bind(q2_name, exchange=exg_name, routing_key=q2_routing_key)
+
+        # Deposit messages in the queues
+        q1_tx_message_bodies = ['q1_message+%s' % (i,)
+                                for i in pika.compat.xrange(100)]
+        for message_body in q1_tx_message_bodies:
+            ch.publish(exg_name, q1_routing_key, body=message_body,
+                       mandatory=True)
+
+        q2_tx_message_bodies = ['q2_message+%s' % (i,)
+                                for i in pika.compat.xrange(150)]
+        for message_body in q2_tx_message_bodies:
+            ch.publish(exg_name, q2_routing_key, body=message_body,
+                       mandatory=True)
+
+        # Create the consumers
+        q1_rx_messages = []
+        q1_consumer_tag = ch.basic_consume(
+            lambda *args: q1_rx_messages.append(args),
+            q1_name,
+            no_ack=False,
+            exclusive=False,
+            arguments=None)
+
+        q2_rx_messages = []
+        q2_consumer_tag = ch.basic_consume(
+            lambda *args: q2_rx_messages.append(args),
+            q2_name,
+            no_ack=False,
+            exclusive=False,
+            arguments=None)
+
+        # Wait for all messages to be delivered
+        while (len(q1_rx_messages) < len(q1_tx_message_bodies) or
+               len(q2_rx_messages) < len(q2_tx_message_bodies)):
+            connection.process_data_events(time_limit=None)
+
+        self.assertEqual(len(q2_rx_messages), len(q2_tx_message_bodies))
+
+        # Verify the messages
+        def validate_messages(rx_messages,
+                              routing_key,
+                              consumer_tag,
+                              tx_message_bodies):
+            self.assertEqual(len(rx_messages), len(tx_message_bodies))
+
+            for msg, expected_body in zip(rx_messages, tx_message_bodies):
+                self.assertIsInstance(msg, tuple)
+                rx_ch, rx_method, rx_properties, rx_body = msg
+                self.assertIs(rx_ch, ch)
+                self.assertIsInstance(rx_method, pika.spec.Basic.Deliver)
+                self.assertEqual(rx_method.consumer_tag, consumer_tag)
+                self.assertFalse(rx_method.redelivered)
+                self.assertEqual(rx_method.exchange, exg_name)
+                self.assertEqual(rx_method.routing_key, routing_key)
+
+                self.assertIsInstance(rx_properties, pika.BasicProperties)
+                self.assertEqual(rx_body, as_bytes(expected_body))
+
+        # Validate q1 consumed messages
+        validate_messages(rx_messages=q1_rx_messages,
+                          routing_key=q1_routing_key,
+                          consumer_tag=q1_consumer_tag,
+                          tx_message_bodies=q1_tx_message_bodies)
+
+        # Validate q2 consumed messages
+        validate_messages(rx_messages=q2_rx_messages,
+                          routing_key=q2_routing_key,
+                          consumer_tag=q2_consumer_tag,
+                          tx_message_bodies=q2_tx_message_bodies)
+
+        # There shouldn't be any more events now
+        self.assertFalse(ch._pending_events)
+
+
 class TestBasicCancelPurgesPendingConsumerCancellationEvt(BlockingTestCaseBase):
 
     def test(self):
@@ -1622,8 +1808,7 @@ class TestBasicPublishWithoutPubacks(BlockingTestCaseBase):
         self.addCleanup(self._connect().channel().queue_delete, q_name)
 
         # Bind the queue to the exchange using routing key
-        frame = ch.queue_bind(q_name, exchange=exg_name,
-                              routing_key=routing_key)
+        ch.queue_bind(q_name, exchange=exg_name, routing_key=routing_key)
 
         # Deposit a message in the queue via basic_publish and mandatory=True
         msg1_headers = dict(
@@ -1638,12 +1823,13 @@ class TestBasicPublishWithoutPubacks(BlockingTestCaseBase):
         # Deposit a message in the queue via basic_publish and mandatory=False
         res = ch.basic_publish(exg_name, routing_key=routing_key,
                                body='via-basic_publish_mandatory=False',
-                               mandatory=True)
+                               mandatory=False)
         self.assertEqual(res, True)
 
         # Wait for the messages to arrive in queue
-        while ch.queue_declare(q_name, passive=True).method.message_count != 2:
-            pass
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
 
         # Create a consumer
         rx_messages = []
@@ -1709,8 +1895,9 @@ class TestBasicPublishWithoutPubacks(BlockingTestCaseBase):
         ch.basic_ack(delivery_tag=rx_method.delivery_tag, multiple=False)
 
         # Verify that the queue is now empty
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
         # Attempt to cosume again with a short timeout
         connection.process_data_events(time_limit=0.005)
@@ -1761,8 +1948,7 @@ class TestPublishFromBasicConsumeCallback(BlockingTestCaseBase):
                          arguments=None)
 
         # Consume from destination queue
-        for _, _, rx_body in ch.consume(dest_q_name,
-                                                       no_ack=True):
+        for _, _, rx_body in ch.consume(dest_q_name, no_ack=True):
             self.assertEqual(rx_body, as_bytes('via-publish'))
             break
         else:
@@ -1977,8 +2163,9 @@ class TestNonPubAckPublishAndConsumeHugeMessage(BlockingTestCaseBase):
         # Verify that the queue is now empty
         ch.close()
         ch = connection.channel()
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestNonPubackPublishAndConsumeManyMessages(BlockingTestCaseBase):
@@ -2034,8 +2221,9 @@ class TestNonPubackPublishAndConsumeManyMessages(BlockingTestCaseBase):
 
         # Verify that the queue is now empty
         ch = connection.channel()
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 0)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
 
 class TestBasicCancelWithNonAckableConsumer(BlockingTestCaseBase):
@@ -2061,16 +2249,18 @@ class TestBasicCancelWithNonAckableConsumer(BlockingTestCaseBase):
         ch.publish(exchange='', routing_key=q_name, body=body2)
 
         # Wait for queue to contain both messages
-        while ch.queue_declare(q_name, passive=True).method.message_count != 2:
-            pass
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
 
         # Create a non-ackable consumer
         consumer_tag = ch.basic_consume(lambda *x: None, q_name, no_ack=True,
                                         exclusive=False, arguments=None)
 
         # Wait for all messages to be sent by broker to client
-        while ch.queue_declare(q_name, passive=True).method.message_count > 0:
-            pass
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
         # Cancel the consumer
         messages = ch.basic_cancel(consumer_tag)
@@ -2088,7 +2278,7 @@ class TestBasicCancelWithNonAckableConsumer(BlockingTestCaseBase):
 
         ch = connection.channel()
 
-        # Verify that the queue is now empty; this validates the multi-ack
+        # Verify that the queue is now empty
         frame = ch.queue_declare(q_name, passive=True)
         self.assertEqual(frame.method.message_count, 0)
 
@@ -2116,16 +2306,18 @@ class TestBasicCancelWithAckableConsumer(BlockingTestCaseBase):
         ch.publish(exchange='', routing_key=q_name, body=body2)
 
         # Wait for queue to contain both messages
-        while ch.queue_declare(q_name, passive=True).method.message_count != 2:
-            pass
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
 
         # Create an ackable consumer
         consumer_tag = ch.basic_consume(lambda *x: None, q_name, no_ack=False,
                                         exclusive=False, arguments=None)
 
         # Wait for all messages to be sent by broker to client
-        while ch.queue_declare(q_name, passive=True).method.message_count > 0:
-            pass
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=0)
 
         # Cancel the consumer
         messages = ch.basic_cancel(consumer_tag)
@@ -2137,9 +2329,10 @@ class TestBasicCancelWithAckableConsumer(BlockingTestCaseBase):
 
         ch = connection.channel()
 
-        # Verify that the queue is now empty; this validates the multi-ack
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 2)
+        # Verify that canceling the ackable consumer restored both messages
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
 
 
 class TestUnackedMessageAutoRestoredToQueueOnChannelClose(BlockingTestCaseBase):
@@ -2187,8 +2380,9 @@ class TestUnackedMessageAutoRestoredToQueueOnChannelClose(BlockingTestCaseBase):
         # Verify that there are two messages in q now
         ch = connection.channel()
 
-        frame = ch.queue_declare(q_name, passive=True)
-        self.assertEqual(frame.method.message_count, 2)
+        self._assert_exact_message_count_with_retries(channel=ch,
+                                                      queue=q_name,
+                                                      expected_count=2)
 
 
 class TestNoAckMessageNotRestoredToQueueOnChannelClose(BlockingTestCaseBase):
@@ -2215,9 +2409,7 @@ class TestNoAckMessageNotRestoredToQueueOnChannelClose(BlockingTestCaseBase):
 
         # Consume, but don't ack
         num_messages = 0
-        for rx_method, _, _ in ch.consume(q_name,
-                                                       no_ack=True,
-                                                       exclusive=False):
+        for rx_method, _, _ in ch.consume(q_name, no_ack=True, exclusive=False):
             num_messages += 1
 
             self.assertEqual(rx_method.delivery_tag, num_messages)
