@@ -5,10 +5,12 @@ from __future__ import print_function
 import array
 from datetime import datetime
 import errno
+from functools import partial
 import logging
 import multiprocessing
 import os
 import socket
+import struct
 import sys
 import threading
 import traceback
@@ -32,7 +34,7 @@ def _trace(fmt, *args):
 
 
 
-class ForwardServer(object):
+class ForwardServer(object):  # pylint: disable=R0902
     """ Implement a TCP/IP forwarding/echo service for testing. Listens for
     an incoming TCP/IP connection, accepts it, then connects to the given
     remote address and forwards data back and forth between the two
@@ -47,7 +49,7 @@ class ForwardServer(object):
 
         with ForwardServer(("localhost", 5672)) as fwd:
             params = pika.ConnectionParameters(
-                host="localhost",
+                host=fwd.server_address[0],
                 port=fwd.server_address[1])
             conn = pika.BlockingConnection(params)
 
@@ -57,14 +59,20 @@ class ForwardServer(object):
         channel = conn.channel()
 
     Echo server example
-        def talk_to_echo_server(port):
-            pass
+        def produce(sock):
+            sock.sendall("12345")
+            sock.shutdown(socket.SHUT_WR)
 
         with ForwardServer(None) as fwd:
-            worker = threading.Thread(target=talk_to_echo_server,
-                                      args=[fwd.server_address[1]])
+            sock = socket.socket()
+            sock.connect(fwd.server_address)
+
+            worker = threading.Thread(target=produce,
+                                      args=[sock])
             worker.start()
-            time.sleep(5)
+
+            data = sock.makefile().read()
+            assert data == "12345", data
 
         worker.join()
 
@@ -73,13 +81,14 @@ class ForwardServer(object):
     _SUBPROC_TIMEOUT = 10
 
 
-    def __init__(self,
+    def __init__(self,  # pylint: disable=R0913
                  remote_addr,
                  remote_addr_family=socket.AF_INET,
                  remote_socket_type=socket.SOCK_STREAM,
                  server_addr=("127.0.0.1", 0),
                  server_addr_family=socket.AF_INET,
-                 server_socket_type=socket.SOCK_STREAM):
+                 server_socket_type=socket.SOCK_STREAM,
+                 local_linger_args=None):
         """
         :param tuple remote_addr: remote server's IP address, whose structure
           depends on remote_addr_family; pair (host-or-ip-addr, port-number).
@@ -96,6 +105,11 @@ class ForwardServer(object):
           socket.AF_UNIX; defaults to socket.AF_INET
         :param server_socket_type: only socket.SOCK_STREAM is supported at this
           time
+        :param tuple local_linger_args: SO_LINGER sockoverride for the local
+          connection sockets, to be configured after connection is accepted.
+          None for default, which is to not change the SO_LINGER option.
+          Otherwise, its a two-tuple, where the first element is the `l_onoff`
+          switch, and the second element is the `l_linger` value, in seconds
         """
         self._logger = logging.getLogger(__name__)
 
@@ -114,6 +128,8 @@ class ForwardServer(object):
         assert server_socket_type == socket.SOCK_STREAM, server_socket_type
         self._server_socket_type = server_socket_type
 
+        self._local_linger_args = local_linger_args
+
         self._subproc = None
 
 
@@ -168,7 +184,7 @@ class ForwardServer(object):
 
         :returns: self
         """
-        q = multiprocessing.Queue()
+        queue = multiprocessing.Queue()
 
         self._subproc = multiprocessing.Process(
             target=_run_server,
@@ -176,18 +192,20 @@ class ForwardServer(object):
                 local_addr=self._server_addr,
                 local_addr_family=self._server_addr_family,
                 local_socket_type=self._server_socket_type,
+                local_linger_args=self._local_linger_args,
                 remote_addr=self._remote_addr,
                 remote_addr_family=self._remote_addr_family,
                 remote_socket_type=self._remote_socket_type,
-                q=q))
+                queue=queue))
         self._subproc.daemon = True
         self._subproc.start()
 
         try:
             # Get server socket info from subprocess
-            self._server_addr_family, self._server_addr = q.get(
+            self._server_addr_family, self._server_addr = queue.get(
                 block=True,
                 timeout=self._SUBPROC_TIMEOUT)
+            queue.close()
         except Exception: # pylint: disable=W0703
             try:
                 self._logger.exception(
@@ -213,7 +231,6 @@ class ForwardServer(object):
         ForwardServer. start()/stop() are alternatives to the context manager
         use case and are mutually exclusive with it.
         """
-        _trace("ForwardServer STOPPING")
         self._logger.info("ForwardServer STOPPING")
 
         try:
@@ -236,26 +253,35 @@ class ForwardServer(object):
 
 
 
-def _run_server(local_addr, local_addr_family, local_socket_type,
-                remote_addr, remote_addr_family, remote_socket_type, q):
+def _run_server(local_addr, local_addr_family, local_socket_type,  # pylint: disable=R0913
+                local_linger_args, remote_addr, remote_addr_family,
+                remote_socket_type, queue):
     """ Run the server; executed in the subprocess
 
     :param local_addr: listening address
     :param local_addr_family: listening address family; one of socket.AF_*
     :param local_socket_type: listening socket type; typically
         socket.SOCK_STREAM
-    :param remote_addr: address of the target server
+    :param tuple local_linger_args: SO_LINGER sockoverride for the local
+        connection sockets, to be configured after connection is accepted.
+        Pass None to not change SO_LINGER. Otherwise, its a two-tuple, where the
+        first element is the `l_onoff` switch, and the second element is the
+        `l_linger` value in seconds
+    :param remote_addr: address of the target server. Pass None to have
+        ForwardServer behave as echo server
     :param remote_addr_family: address family for connecting to target server;
         one of socket.AF_*
     :param remote_socket_type: socket type for connecting to target server;
         typically socket.SOCK_STREAM
-    :param multiprocessing.Queue q: queue for depositing the forwarding server's
-        actual listening socket address family and bound address. The parent
-        process waits for this.
+    :param multiprocessing.Queue queue: queue for depositing the forwarding
+        server's actual listening socket address family and bound address. The
+        parent process waits for this.
     """
 
     # NOTE: We define _ThreadedTCPServer class as a closure in order to
     # override some of its class members dynamically
+    # NOTE: we add `object` to the base classes because `_ThreadedTCPServer`
+    # isn't derived from `object`, which prevents `super` from working properly
     class _ThreadedTCPServer(SocketServer.ThreadingMixIn,
                              SocketServer.TCPServer,
                              object):
@@ -267,38 +293,35 @@ def _run_server(local_addr, local_addr_family, local_socket_type,
         allow_reuse_address = True
 
 
-        def __init__(self,
-                     remote_addr,
-                     remote_addr_family,
-                     remote_socket_type):
-            self.remote_addr = remote_addr
-            self.remote_addr_family = remote_addr_family
-            self.remote_socket_type = remote_socket_type
+        def __init__(self):
+
+            handler_class_factory = partial(
+                _TCPHandler,
+                local_linger_args=local_linger_args,
+                remote_addr=remote_addr,
+                remote_addr_family=remote_addr_family,
+                remote_socket_type=remote_socket_type)
 
             super(_ThreadedTCPServer, self).__init__(
                 local_addr,
-                _TCPHandler,
+                handler_class_factory,
                 bind_and_activate=True)
 
 
-    server = _ThreadedTCPServer(remote_addr,
-                                remote_addr_family,
-                                remote_socket_type)
+    server = _ThreadedTCPServer()
 
     # Send server socket info back to parent process
-    q.put([server.socket.family, server.server_address])
-    q.close()
+    queue.put([server.socket.family, server.server_address])
 
-##    # Validate server's socket fileno
-##    _trace("Checking server fd=%s after q.put", server.socket.fileno())
-##    fcntl.fcntl(server.socket.fileno(), fcntl.F_GETFL)
-##    _trace("Server fd is OK after q.put")
+    queue.close()
 
     server.serve_forever()
 
 
 
-class _TCPHandler(SocketServer.StreamRequestHandler):
+# NOTE: we add `object` to the base classes because `StreamRequestHandler` isn't
+# derived from `object`, which prevents `super` from working properly
+class _TCPHandler(SocketServer.StreamRequestHandler, object):
     """TCP/IP session handler instantiated by TCPServer upon incoming
     connection. Implements forwarding/echo of the incoming connection.
     """
@@ -306,56 +329,93 @@ class _TCPHandler(SocketServer.StreamRequestHandler):
     _SOCK_RX_BUF_SIZE = 16 * 1024
 
 
-    def handle(self):
-        try:
-            local_sock = self.connection
+    def __init__(self,  # pylint: disable=R0913
+                 request,
+                 client_address,
+                 server,
+                 local_linger_args,
+                 remote_addr,
+                 remote_addr_family,
+                 remote_socket_type):
+        """
+        :param request: for super
+        :param client_address: for super
+        "paarm server:  for super
+        :param tuple local_linger_args: SO_LINGER sockoverride for the local
+            connection sockets, to be configured after connection is accepted.
+            Pass None to not change SO_LINGER. Otherwise, its a two-tuple, where
+            the first element is the `l_onoff` switch, and the second element is
+            the `l_linger` value in seconds
+        :param remote_addr: address of the target server. Pass None to have
+            ForwardServer behave as echo server.
+        :param remote_addr_family: address family for connecting to target
+            server; one of socket.AF_*
+        :param remote_socket_type: socket type for connecting to target server;
+            typically socket.SOCK_STREAM
+        :param **kwargs: kwargs for super class
+        """
+        self._local_linger_args = local_linger_args
+        self._remote_addr = remote_addr
+        self._remote_addr_family = remote_addr_family
+        self._remote_socket_type = remote_socket_type
 
-            if self.server.remote_addr is not None:
-                # Forwarding set-up
-                remote_dest_sock = remote_src_sock = socket.socket(
-                    family=self.server.remote_addr_family,
-                    type=self.server.remote_socket_type,
-                    proto=socket.IPPROTO_IP)
-                remote_dest_sock.connect(self.server.remote_addr)
-                _trace("%s _TCPHandler connected to remote %s",
-                       datetime.utcnow(), remote_dest_sock.getpeername())
-            else:
-                # Echo set-up
-                remote_dest_sock, remote_src_sock = socket_pair()
+        super(_TCPHandler, self).__init__(request=request,
+                                          client_address=client_address,
+                                          server=server)
+
+
+    def handle(self):  # pylint: disable=R0912
+        """Connect to remote and forward data between local and remote"""
+        local_sock = self.connection
+
+        if self._local_linger_args is not None:
+            # Set SO_LINGER socket options on local socket
+            l_onoff, l_linger = self._local_linger_args
+            local_sock.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER,
+                                  struct.pack('ii', l_onoff, l_linger))
+
+        if self._remote_addr is not None:
+            # Forwarding set-up
+            remote_dest_sock = remote_src_sock = socket.socket(
+                family=self._remote_addr_family,
+                type=self._remote_socket_type,
+                proto=socket.IPPROTO_IP)
+            remote_dest_sock.connect(self._remote_addr)
+            _trace("%s _TCPHandler connected to remote %s",
+                   datetime.utcnow(), remote_dest_sock.getpeername())
+        else:
+            # Echo set-up
+            remote_dest_sock, remote_src_sock = socket_pair()
+
+        try:
+            local_forwarder = threading.Thread(
+                target=self._forward,
+                args=(local_sock, remote_dest_sock,))
+            local_forwarder.setDaemon(True)
+            local_forwarder.start()
 
             try:
-                local_forwarder = threading.Thread(
-                    target=self._forward,
-                    args=(local_sock, remote_dest_sock,))
-                local_forwarder.setDaemon(True)
-                local_forwarder.start()
-
-                try:
-                    self._forward(remote_src_sock, local_sock)
-                finally:
-                    # Wait for local forwarder thread to exit
-                    local_forwarder.join()
+                self._forward(remote_src_sock, local_sock)
             finally:
+                # Wait for local forwarder thread to exit
+                local_forwarder.join()
+        finally:
+            try:
                 try:
-                    try:
-                        _safe_shutdown_socket(remote_dest_sock,
-                                              socket.SHUT_RDWR)
-                    finally:
-                        if remote_src_sock is not remote_dest_sock:
-                            _safe_shutdown_socket(remote_src_sock,
-                                                  socket.SHUT_RDWR)
+                    _safe_shutdown_socket(remote_dest_sock,
+                                          socket.SHUT_RDWR)
                 finally:
-                    remote_dest_sock.close()
                     if remote_src_sock is not remote_dest_sock:
-                        remote_src_sock.close()
-        except:
-            _trace("handle failed:\n%s", "".join(traceback.format_exc()))
-            raise
+                        _safe_shutdown_socket(remote_src_sock,
+                                              socket.SHUT_RDWR)
+            finally:
+                remote_dest_sock.close()
+                if remote_src_sock is not remote_dest_sock:
+                    remote_src_sock.close()
 
 
-    def _forward(self, src_sock, dest_sock):
+    def _forward(self, src_sock, dest_sock): # pylint: disable=R0912
         """Forward from src_sock to dest_sock"""
-
         src_peername = src_sock.getpeername()
 
         _trace("%s forwarding from %s to %s", datetime.utcnow(),
@@ -371,17 +431,17 @@ class _TCPHandler(SocketServer.StreamRequestHandler):
             while True:
                 try:
                     nbytes = src_sock.recv_into(rx_buf)
-                except socket.error as e:
-                    if e.errno == errno.EINTR:
+                except socket.error as exc:
+                    if exc.errno == errno.EINTR:
                         continue
-                    elif e.errno == errno.ECONNRESET:
+                    elif exc.errno == errno.ECONNRESET:
                         # Source peer forcibly closed connection
                         _trace("%s errno.ECONNRESET from %s",
                                datetime.utcnow(), src_peername)
                         break
                     else:
                         _trace("%s Unexpected errno=%s from %s\n%s",
-                               datetime.utcnow(), e.errno, src_peername,
+                               datetime.utcnow(), exc.errno, src_peername,
                                "".join(traceback.format_stack()))
                         raise
 
@@ -392,14 +452,14 @@ class _TCPHandler(SocketServer.StreamRequestHandler):
 
                 try:
                     dest_sock.sendall(buffer(rx_buf, 0, nbytes))
-                except socket.error as e:
-                    if e.errno == errno.EPIPE:
+                except socket.error as exc:
+                    if exc.errno == errno.EPIPE:
                         # Destination peer closed its end of the connection
                         _trace("%s Destination peer %s closed its end of "
                                "the connection: errno.EPIPE",
                                datetime.utcnow(), dest_sock.getpeername())
                         break
-                    elif e.errno == errno.ECONNRESET:
+                    elif exc.errno == errno.ECONNRESET:
                         # Destination peer forcibly closed connection
                         _trace("%s Destination peer %s forcibly closed "
                                "connection: errno.ECONNRESET",
@@ -408,7 +468,7 @@ class _TCPHandler(SocketServer.StreamRequestHandler):
                     else:
                         _trace(
                             "%s Unexpected errno=%s in sendall to %s\n%s",
-                            datetime.utcnow(), e.errno,
+                            datetime.utcnow(), exc.errno,
                             dest_sock.getpeername(),
                             "".join(traceback.format_stack()))
                         raise
@@ -450,8 +510,8 @@ def echo(port=0):
         while True:
             try:
                 data = sock.recv(4 * 1024) # pylint: disable=E1101
-            except socket.error as e:
-                if e.errno == errno.EINTR:
+            except socket.error as exc:
+                if exc.errno == errno.EINTR:
                     continue
                 else:
                     raise
@@ -473,8 +533,8 @@ def _safe_shutdown_socket(sock, how=socket.SHUT_RDWR):
     """
     try:
         sock.shutdown(how)
-    except socket.error as e:
-        if e.errno != errno.ENOTCONN:
+    except socket.error as exc:
+        if exc.errno != errno.ENOTCONN:
             raise
 
 
