@@ -29,4 +29,7 @@ class TornadoConnectionTests(unittest.TestCase):
     @mock.patch('pika.adapters.base_connection.BaseConnection.__init__')
     def test_tornado_connection_call_parent(self, mock_init):
         obj = tornado_connection.TornadoConnection()
-        mock_init.called_once_with(None, None, False)
+        mock_init.assert_called_once_with(
+            None, None, None, None,
+            tornado_connection.ioloop.IOLoop.instance(),
+            False)
